#
# Lines starting with number sign are comments, which are ignored by R
#
# A monte Carlo Simulation runs a function that calculates a statistic and this is
# repeated multiple times to see the behavior of the statistic over replications.
#
# The function below generates three multivariate normal variables and then generates a
# fourth variable which is a weighted sum of the three variables plus a normally 
# distributed error term. The data are then used to estimate a regression model and
# results from all replications are collected as a matrix
#


# We need the MASS library for data generation
if("MASS" %in% rownames(installed.packages()) == FALSE) install.packages("MASS")
library(MASS)

# Needed for unmatrix function
if("gdata" %in% rownames(installed.packages()) == FALSE) install.packages("gdata")
library(gdata)

# Needed for describe
if("psych" %in% rownames(installed.packages()) == FALSE) install.packages("psych")
library(psych)

OLS <- function(){
  
  V <- matrix(c(1, .3, .3, .3, 1, .3, .3, .3, 1), 3, 3)
  data <- mvrnorm(200, rep(0,3), V)
  
  x1 <- data[,1]
  x2 <- data[,2]
  x3 <- data[,3]
  
  # Genrate the dependent variable using an equation
  
  y <- 1*x1 + 2*x2 + 3*x3 + 5*rnorm(200)
  
  # Estimate regression and extract results
  fit <- lm(y ~ x1 + x2 + x3)
  
  # The estimates, SEs, and other results are returned as a matrix
  # get the first two columns (estimates and SEs) and return as vector
  
  unmatrix(summary(fit)$coef[,1:2])
}

# Do a simulation with the function. The resulting array needs to be transposed
# and for convenience

MonteCarloReplications <- t(replicate(1000, OLS()))

# Print summary statistics of the 1000 replications. This shows that the estimates are
# correct on average (unbiased) and that standard errors are unbiased estimates of the
# standard deviations of the estimates

print(describe(MonteCarloReplications), digits = 3)

# Two plots side by side
par(mfrow = c(1,2))

# Show that estimates of X1 are normally distributed around the true value
plot(density(MonteCarloReplications[,"x1:Estimate"]), main = "Estimates")

# Add normal distribution as a reference
curve(dnorm(x, 
            mean = mean(MonteCarloReplications[,"x1:Estimate"]),
            sd = sd(MonteCarloReplications[,"x1:Estimate"])),
      add = TRUE, col="red", lty = 2)

# Vertical line at the population value
abline(v=1, col = "red")


# Show that standard errors are unbiased estimates of the standard deviation of the
# estimates

plot(density(MonteCarloReplications[,"x1:Std. Error"]), main = "Standard errors")

# Add normal distribution as a reference
curve(dnorm(x, 
            mean = mean(MonteCarloReplications[,"x1:Std. Error"]),
            sd = sd(MonteCarloReplications[,"x1:Std. Error"])),
      add = TRUE, col="red", lty = 2)
# Vertical line at the true SD
abline(v=sd(MonteCarloReplications[,"x1:Estimate"]), col = "red")


